// v3-follow.js --  create a unity vector that follows the input vector with constant rate
//
// Author: Hans Wilmers (hanswil@notam02.no)
// Date: 2009-12-14
// License: This code is realeased under GPL Version 3
//
//  This code works only with input vectors with 3 elements.
// 
//
// Pseudocode:
//
//  if |result| < 0.5, then
//     result = input
// else
//     turn result with constant rate towards input
//     scale the result to unity length
// endif
// 
// Remarks:
// The rotation is implemented using a simplification that is valid for small angular rates.
// For big rates, this results in the need for some more iterations.
//


// inlets and outlets
inlets = 1;
outlets = 1;

// global variables
var rate = 3.1415/180; // angular rate, default=1 degree
var x = 0;
var y = 0;
var z = 0;

function vector_crossproduct(result,a,b)
{
   result[0] = a[1]*b[2]-a[2]*b[1];    
   result[1] = a[2]*b[0]-a[0]*b[2];    
   result[2] = a[0]*b[1]-a[1]*b[0];    
}

function vector_add(a,b)
{
    for(var i=0; i<3; i++)
        a[i] += b[i];
}

function vector_length(a)
{
    var len = 0;
    for(var i=0; i<a.length;i++)
        len += a[i]*a[i];

    if(len==0)
        return 0;
    else
        return Math.sqrt(len);
}

function vector_post(a)
{
    for(var i=0; i<a.length;i++)
        post(a[i]); 
    post();
}

function vector_scale(a,scalar)
{
    for(var i=0; i<a.length;i++)
        a[i] *= scalar;
}

// float -- set rate
function msg_float(r)
{
   rate = r;
}


function list(val)
{
    if(arguments.length==3)
	{
	    var vFollow = new Array(3);
	    vFollow[0] = x;
	    vFollow[1] = y;
	    vFollow[2] = z;

	    if(vector_length(vFollow)<0.5)
		{
		    // set to initial value
		    for(var i=0; i<3; i++)
			vFollow[i] = arguments[i];
		}
	    else
		{
		    var vRotate = new Array(3);
		    var vTmp = new Array(3);

		    // calculate rotation vector
		    vector_crossproduct(vRotate,arguments,vFollow);

		    // replace all elements with sign*rate unless abs(element)<rate
		    for(var i=0; i<3; i++)
			{
			    if(Math.abs(vRotate[i] )>rate)
				{
				    if(vRotate[i]>0)
					vRotate[i] = rate;
				    else
					vRotate[i] = -rate;
				}
			}

		    // apply simplified rotation matrix: identity+(crossproduct with vRotate)
		    vector_crossproduct(vTmp,vFollow,vRotate);
		    vector_add(vFollow,vTmp);
		}

	    // scale result to unity length
	    vector_scale(vFollow,1/vector_length(vFollow));

	    // store result
	    x =  vFollow[0];
	    y =  vFollow[1];
	    z =  vFollow[2];

	    // trigger output
	    bang();
	}
    else
	{
	    // ERROR!
	    post("Error: Expecting vector with 3 elements");
	    post();
	}
}

// bang -- post the current result
function bang()
{
    outlet(0,[x,y,z]);
}